<?php

/**
 * Output a select input box.
 *
 * @param array $field Data about the field to render.
 */
if (!function_exists('helloprint_wp_select')) {
	function helloprint_wp_select($field)
	{
		global $thepostid, $post;

		$thepostid = empty($thepostid) ? $post->ID : $thepostid;
		$field     = wp_parse_args(
			$field,
			array(
				'class'             => 'select short',
				'style'             => '',
				'wrapper_class'     => '',
				'value'             => get_post_meta($thepostid, $field['id'], true),
				'name'              => $field['id'],
				'desc_tip'          => false,
				'custom_attributes' => array(),
			)
		);

		$wrapper_attributes = array(
			'class' => $field['wrapper_class'] . " form-field {$field['id']}_field",
		);

		$label_attributes = array(
			'for' => $field['id'],
		);

		$field_attributes          = (array) $field['custom_attributes'];
		$field_attributes['style'] = $field['style'];
		$field_attributes['id']    = $field['id'];
		$field_attributes['name']  = $field['name'];
		$field_attributes['class'] = $field['class'];

		$tooltip     = !empty($field['description']) && false !== $field['desc_tip'] ? $field['description'] : '';
		$description = !empty($field['description']) && false === $field['desc_tip'] ? $field['description'] : '';
?>
		<p <?php echo wc_implode_html_attributes($wrapper_attributes); // WPCS: XSS ok. 
			?>>
			<label <?php echo wc_implode_html_attributes($label_attributes); // WPCS: XSS ok. 
					?>><?php echo wp_kses_post($field['label']); ?></label>
			<?php if ($tooltip) : ?>
				<?php echo wc_help_tip($tooltip); // WPCS: XSS ok. 
				?>
			<?php endif; ?>
			<select <?php echo wc_implode_html_attributes($field_attributes); // WPCS: XSS ok. 
					?>>
				<?php
				foreach ($field['options'] as $key => $value) {
					if (!is_array($value)) {
						echo '<option value="' . esc_attr($key) . '"' . esc_html(wc_selected($key, $field['value'])) . '>' . esc_html($value) . '</option>';
					} else {
						echo '<optgroup label="' . esc_attr($key) . '">';
						foreach ($value as $k => $val) {
							echo '<option value="' . esc_attr($k) . '"' . esc_html(wc_selected($k, $field['value'])) . '>' . esc_html($val) . '</option>';
						}
						echo '</optgroup>';
					}
				}
				?>
			</select>
			<?php if ($description) : ?>
				<span class="description"><?php echo wp_kses_post($description); ?></span>
			<?php endif; ?>
		</p>
<?php
	}
}


if (!function_exists('all_existing_hello_print_product_ids')) {
	function all_existing_hello_print_product_ids()
	{
		global $wpdb;
		$hp_ids = [];
		$table = $wpdb->prefix . 'postmeta';
		$query = 'SELECT meta_value from ' . $table . ' where meta_key="helloprint_external_product_id"';
		$results = $wpdb->get_results($query);
		foreach ($results as $res) {
			$hp_ids[] = $res->meta_value;
		}
		return $hp_ids;
	}
}


if (!function_exists("_helloprint_get_graphic_design_price")) {
	function _helloprint_get_graphic_design_price($productId = null)
	{
		$product_graphic_design_price = 0;
		$enable_product_design = get_post_meta($productId, 'helloprint_product_graphic_design_fee', true);
		if ($enable_product_design == 1) {
			$enable_product_design = true;
			$product_graphic_design_price = get_post_meta($productId, 'helloprint_product_graphic_design_price', true);
		} else if ($enable_product_design == -1) {
			$enable_product_design = false;
			$product_graphic_design_price = 0;
		} else {
			$enable_product_design = get_option("helloprint_enable_global_graphic_design");
			$product_graphic_design_price = ($enable_product_design) ? get_option("helloprint_global_graphic_design_price") : 0;
		}
		$enable_product_design = ($product_graphic_design_price > 0) ? $enable_product_design : false;
		return ['enabled' => $enable_product_design, 'price' => $product_graphic_design_price];
	}
}


if (!function_exists("helloprint_get_max_file_upload_size")) {
	function helloprint_get_max_file_upload_size()
	{
		$size = wp_max_upload_size();
		$base = log($size) / log(1024);
		$suffix = array("", "KB", "MB", "GB", "TB")[floor($base)];
		return pow(1024, $base - floor($base)) . $suffix;
	}
}

if (!function_exists("helloprint_get_checkout_country_code")) {
	function helloprint_get_checkout_country_code()
	{

		$customer = WC()->session->get('customer');
		$shipping_country = (!empty($customer['shipping_country'])) ? esc_html($customer['shipping_country']) : '';
		$billing_country = (!empty($customer['billing_country'])) ? esc_html($customer['billing_country']) : '';
		$current_user = wp_get_current_user();
		$returnCountry = '';
		if (!empty($shipping_country) || !empty($billing_country)) {
			$returnCountry = ($shipping_country) ?? $billing_country;
		} else if (!empty($current_user->billing_country)) {
			$returnCountry = $current_user->billing_country;
		} else {
			$countries_obj   = new WC_Countries();
			$returnCountry = $countries_obj->get_base_country();
		}
		return $returnCountry;
	}
}

if (!function_exists("add_helloprint_flash_notice")) {
	function add_helloprint_flash_notice($notice = "", $type = "warning", $dismissible = true)
	{
		// Here we return the notices saved on our option, if there are not notices, then an empty array is returned
		$notices = get_option("helloprint_flash_notices", array());

		$dismissible_text = ($dismissible) ? "is-dismissible" : "";

		// We add our new notice.
		array_push($notices, array(
			"notice" => $notice,
			"type" => $type,
			"dismissible" => $dismissible_text
		));
		delete_option("helloprint_flash_notices");
		// Then we update the option with our notices array
		add_option("helloprint_flash_notices", $notices);
	}
}

if (!function_exists("helloprint_get_data_from_url")) {
	function helloprint_get_data_from_url($url = "")
	{
		$ch = curl_init();
		curl_setopt($ch, CURLOPT_URL, $url);
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
		$data = curl_exec($ch);
		curl_close($ch);
		return $data;
	}
}

if (!function_exists("hp_is_pitchprint_plugin_active")) {
	function hp_is_pitchprint_plugin_active($response = "active")
	{
		include_once ABSPATH . 'wp-admin/includes/plugin.php';
		$activePlugins = get_option('active_plugins', array());
		foreach ($activePlugins as $plugin) {
			if (strpos($plugin, "pitchprint.php") !== false) {
				if ($response == "root_url") {
					$pluginNameArray = explode("/", $plugin);
					return $pluginNameArray[0];
				}
				return true;
			}
		}
		return false;
	}
}

if (!function_exists("_get_helloprint_version")) {
	function _get_helloprint_version()
	{
		global $wp_version;
		$version = $wp_version;
		$plugin_base_arr = explode('/', plugin_basename(__FILE__));
		if (isset($plugin_base_arr)) {
			$base_name = $plugin_base_arr[0];
			foreach (\get_plugins() as $key => $value) {
				if (strpos($key, $base_name) === 0) {
					$version = $value['Version'];
				}
			}
		}
		return $version;
	}
}


